#define _XOPEN_SOURCE 1
#define _XOPEN_SOURCE_EXTENDED 1
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

static char const prog[]="grabconsole";

static char const dev_console[]="/dev/console";

int main(int argc, char** argv) {
  int fd;

  if (argc!=2)
    strerr_die4x(111, prog, usage, prog, " file");

  if ('0'<=argv[1][0] && argv[1][0]<='9') {
    unsigned long scan;
    unsigned int len;
    len=scan_ulong(argv[1], &scan);
    if (len==0 || argv[1][len]!='\0')
      strerr_die6x(111, prog, fatal, arg_fd, err_malformed, err_, argv[1]);
    if (scan>INT_MAX)
      strerr_die5x(100, prog, fatal, arg_fd, err_oflow, t_int);
    fd=scan;
  } else {
    fd=open(argv[1], O_RDONLY, 0);
    if (fd<0)
      strerr_die6sys(100, prog, fatal, err_unable, err_open, argv[1], err_);
  }

  switch (grabconsole(fd)) {
    case grabconsole_success:
      _exit(0);
    case grabconsole_ioctl:
      strerr_die5sys(100, prog, fatal, err_unable, "do ioctl", err_);
    case grabconsole_devconsole_open: {
      int status=100;
      if (errno==ENFILE || errno==ENOMEM) status=111;
      strerr_die6sys(status, prog, fatal, err_unable, err_open, dev_console,
                     err_);
    }
    case grabconsole_devconsole_close:
      strerr_die6sys(100, prog, fatal, err_unable, err_close, dev_console,
                     err_);
    default:
      strerr_die4sys(111, prog, fatal, err_unknown, err_);
  }
}
